package com.sun.media.sound;

import javax.sound.midi.Instrument;
import javax.sound.midi.Patch;
import java.util.Arrays;
import javax.sound.midi.SoundbankResource;
import java.util.Map;
import java.io.OutputStream;
import java.util.Iterator;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.io.File;
import javax.sound.midi.Soundbank;

public final class SF2Soundbank implements Soundbank
{
    int major;
    int minor;
    String targetEngine;
    String name;
    String romName;
    int romVersionMajor;
    int romVersionMinor;
    String creationDate;
    String engineers;
    String product;
    String copyright;
    String comments;
    String tools;
    private ModelByteBuffer sampleData;
    private ModelByteBuffer sampleData24;
    private File sampleFile;
    private boolean largeFormat;
    private final List<SF2Instrument> instruments;
    private final List<SF2Layer> layers;
    private final List<SF2Sample> samples;
    
    public SF2Soundbank() {
        this.major = 2;
        this.minor = 1;
        this.targetEngine = "EMU8000";
        this.name = "untitled";
        this.romName = null;
        this.romVersionMajor = -1;
        this.romVersionMinor = -1;
        this.creationDate = null;
        this.engineers = null;
        this.product = null;
        this.copyright = null;
        this.comments = null;
        this.tools = null;
        this.sampleData = null;
        this.sampleData24 = null;
        this.sampleFile = null;
        this.largeFormat = false;
        this.instruments = new ArrayList();
        this.layers = new ArrayList();
        this.samples = new ArrayList();
    }
    
    public SF2Soundbank(URL url) throws IOException {
        this.major = 2;
        this.minor = 1;
        this.targetEngine = "EMU8000";
        this.name = "untitled";
        this.romName = null;
        this.romVersionMajor = -1;
        this.romVersionMinor = -1;
        this.creationDate = null;
        this.engineers = null;
        this.product = null;
        this.copyright = null;
        this.comments = null;
        this.tools = null;
        this.sampleData = null;
        this.sampleData24 = null;
        this.sampleFile = null;
        this.largeFormat = false;
        this.instruments = new ArrayList();
        this.layers = new ArrayList();
        this.samples = new ArrayList();
        InputStream openStream = url.openStream();
        try {
            this.readSoundbank(openStream);
        }
        finally {
            openStream.close();
        }
    }
    
    public SF2Soundbank(File file) throws IOException {
        this.major = 2;
        this.minor = 1;
        this.targetEngine = "EMU8000";
        this.name = "untitled";
        this.romName = null;
        this.romVersionMajor = -1;
        this.romVersionMinor = -1;
        this.creationDate = null;
        this.engineers = null;
        this.product = null;
        this.copyright = null;
        this.comments = null;
        this.tools = null;
        this.sampleData = null;
        this.sampleData24 = null;
        this.sampleFile = null;
        this.largeFormat = false;
        this.instruments = new ArrayList();
        this.layers = new ArrayList();
        this.samples = new ArrayList();
        this.largeFormat = true;
        this.sampleFile = file;
        FileInputStream fileInputStream = new FileInputStream(file);
        try {
            this.readSoundbank(fileInputStream);
        }
        finally {
            fileInputStream.close();
        }
    }
    
    public SF2Soundbank(InputStream inputStream) throws IOException {
        this.major = 2;
        this.minor = 1;
        this.targetEngine = "EMU8000";
        this.name = "untitled";
        this.romName = null;
        this.romVersionMajor = -1;
        this.romVersionMinor = -1;
        this.creationDate = null;
        this.engineers = null;
        this.product = null;
        this.copyright = null;
        this.comments = null;
        this.tools = null;
        this.sampleData = null;
        this.sampleData24 = null;
        this.sampleFile = null;
        this.largeFormat = false;
        this.instruments = new ArrayList();
        this.layers = new ArrayList();
        this.samples = new ArrayList();
        this.readSoundbank(inputStream);
    }
    
    private void readSoundbank(InputStream inputStream) throws IOException {
        RIFFReader riffReader = new RIFFReader(inputStream);
        if (!riffReader.getFormat().equals("RIFF")) {
            throw new RIFFInvalidFormatException("Input stream is not a valid RIFF stream!");
        }
        if (!riffReader.getType().equals("sfbk")) {
            throw new RIFFInvalidFormatException("Input stream is not a valid SoundFont!");
        }
        while (riffReader.hasNextChunk()) {
            RIFFReader nextChunk = riffReader.nextChunk();
            if (nextChunk.getFormat().equals("LIST")) {
                if (nextChunk.getType().equals("INFO")) {
                    this.readInfoChunk(nextChunk);
                }
                if (nextChunk.getType().equals("sdta")) {
                    this.readSdtaChunk(nextChunk);
                }
                if (nextChunk.getType().equals("pdta")) {
                    this.readPdtaChunk(nextChunk);
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
        return;
    }
    
    private void readInfoChunk(RIFFReader riffReader) throws IOException {
        while (riffReader.hasNextChunk()) {
            RIFFReader nextChunk = riffReader.nextChunk();
            String format = nextChunk.getFormat();
            if (format.equals("ifil")) {
                this.major = nextChunk.readUnsignedShort();
                this.minor = nextChunk.readUnsignedShort();
            }
            else if (format.equals("isng")) {
                this.targetEngine = nextChunk.readString(nextChunk.available());
            }
            else if (format.equals("INAM")) {
                this.name = nextChunk.readString(nextChunk.available());
            }
            else if (format.equals("irom")) {
                this.romName = nextChunk.readString(nextChunk.available());
            }
            else if (format.equals("iver")) {
                this.romVersionMajor = nextChunk.readUnsignedShort();
                this.romVersionMinor = nextChunk.readUnsignedShort();
            }
            else if (format.equals("ICRD")) {
                this.creationDate = nextChunk.readString(nextChunk.available());
            }
            else if (format.equals("IENG")) {
                this.engineers = nextChunk.readString(nextChunk.available());
            }
            else if (format.equals("IPRD")) {
                this.product = nextChunk.readString(nextChunk.available());
            }
            else if (format.equals("ICOP")) {
                this.copyright = nextChunk.readString(nextChunk.available());
            }
            else if (format.equals("ICMT")) {
                this.comments = nextChunk.readString(nextChunk.available());
            }
            else if (format.equals("ISFT")) {
                this.tools = nextChunk.readString(nextChunk.available());
            }
            else {
                continue;
            }
        }
    }
    
    private void readSdtaChunk(RIFFReader riffReader) throws IOException {
        while (riffReader.hasNextChunk()) {
            RIFFReader nextChunk = riffReader.nextChunk();
            if (nextChunk.getFormat().equals("smpl")) {
                if (!this.largeFormat) {
                    byte[] array = new byte[nextChunk.available()];
                    int i = 0;
                    int available = nextChunk.available();
                    while (i != available) {
                        if (available - i > 65536) {
                            nextChunk.readFully(array, i, 65536);
                            i += 65536;
                        }
                        else {
                            nextChunk.readFully(array, i, available - i);
                            i = available;
                        }
                    }
                    this.sampleData = new ModelByteBuffer(array);
                }
                else {
                    this.sampleData = new ModelByteBuffer(this.sampleFile, nextChunk.getFilePointer(), (long)nextChunk.available());
                }
            }
            if (nextChunk.getFormat().equals("sm24")) {
                if (!this.largeFormat) {
                    byte[] array2 = new byte[nextChunk.available()];
                    int j = 0;
                    int available2 = nextChunk.available();
                    while (j != available2) {
                        if (available2 - j > 65536) {
                            nextChunk.readFully(array2, j, 65536);
                            j += 65536;
                        }
                        else {
                            nextChunk.readFully(array2, j, available2 - j);
                            j = available2;
                        }
                    }
                    this.sampleData24 = new ModelByteBuffer(array2);
                }
                else {
                    this.sampleData24 = new ModelByteBuffer(this.sampleFile, nextChunk.getFilePointer(), (long)nextChunk.available());
                }
            }
            else {
                continue;
            }
        }
    }
    
    private void readPdtaChunk(RIFFReader riffReader) throws IOException {
        ArrayList<Object> list = new ArrayList();
        ArrayList<Integer> list2 = new ArrayList();
        ArrayList<Object> list3 = new ArrayList();
        ArrayList<Object> list4 = new ArrayList();
        ArrayList<Object> list5 = new ArrayList();
        ArrayList<Integer> list6 = new ArrayList();
        ArrayList<Object> list7 = new ArrayList();
        ArrayList<Object> list8 = new ArrayList();
        while (riffReader.hasNextChunk()) {
            RIFFReader nextChunk = riffReader.nextChunk();
            String format = nextChunk.getFormat();
            if (format.equals("phdr")) {
                if (nextChunk.available() % 38 != 0) {
                    throw new RIFFInvalidDataException();
                }
                for (int n15 = nextChunk.available() / 38, n16 = 0; n16 < n15; ++n16) {
                    SF2Instrument o7 = new SF2Instrument(this);
                    o7.name = nextChunk.readString(20);
                    o7.preset = nextChunk.readUnsignedShort();
                    o7.bank = nextChunk.readUnsignedShort();
                    list2.add(Integer.valueOf(nextChunk.readUnsignedShort()));
                    o7.library = nextChunk.readUnsignedInt();
                    o7.genre = nextChunk.readUnsignedInt();
                    o7.morphology = nextChunk.readUnsignedInt();
                    list.add(o7);
                    if (n16 != n15 - 1) {
                        this.instruments.add(o7);
                    }
                }
            }
            else if (format.equals("pbag")) {
                if (nextChunk.available() % 4 != 0) {
                    throw new RIFFInvalidDataException();
                }
                int n10 = nextChunk.available() / 4;
                int unsignedShort9 = nextChunk.readUnsignedShort();
                int unsignedShort10 = nextChunk.readUnsignedShort();
                while (list3.size() < unsignedShort9) {
                    list3.add(null);
                }
                while (list4.size() < unsignedShort10) {
                    list4.add(null);
                }
                --n10;
                if (list2.isEmpty()) {
                    throw new RIFFInvalidDataException();
                }
                int intValue2 = (int)list2.get(0);
                int n11 = 0;
                while (n11 < intValue2) {
                    if (n10 == 0) {
                        throw new RIFFInvalidDataException();
                    }
                    int unsignedShort11 = nextChunk.readUnsignedShort();
                    int unsignedShort12 = nextChunk.readUnsignedShort();
                    while (list3.size() < unsignedShort11) {
                        list3.add(null);
                    }
                    while (list4.size() < unsignedShort12) {
                        list4.add(null);
                    }
                    --n10;
                    ++n11;
                }
                for (int n12 = 0; n12 < list2.size() - 1; ++n12) {
                    int n13 = (int)list2.get(n12 + 1) - (int)list2.get(n12);
                    SF2Instrument sf2Instrument = (SF2Instrument)list.get(n12);
                    int n14 = 0;
                    while (n14 < n13) {
                        if (n10 == 0) {
                            throw new RIFFInvalidDataException();
                        }
                        int unsignedShort13 = nextChunk.readUnsignedShort();
                        int unsignedShort14 = nextChunk.readUnsignedShort();
                        SF2InstrumentRegion o6 = new SF2InstrumentRegion();
                        sf2Instrument.regions.add(o6);
                        while (list3.size() < unsignedShort13) {
                            list3.add(o6);
                        }
                        while (list4.size() < unsignedShort14) {
                            list4.add(o6);
                        }
                        --n10;
                        ++n14;
                    }
                }
            }
            else if (format.equals("pmod")) {
                for (int n9 = 0; n9 < list4.size(); ++n9) {
                    SF2Modulator o5 = new SF2Modulator();
                    o5.sourceOperator = nextChunk.readUnsignedShort();
                    o5.destinationOperator = nextChunk.readUnsignedShort();
                    o5.amount = nextChunk.readShort();
                    o5.amountSourceOperator = nextChunk.readUnsignedShort();
                    o5.transportOperator = nextChunk.readUnsignedShort();
                    SF2InstrumentRegion sf2InstrumentRegion2 = (SF2InstrumentRegion)list4.get(n9);
                    if (sf2InstrumentRegion2 != null) {
                        sf2InstrumentRegion2.modulators.add(o5);
                    }
                }
            }
            else if (format.equals("pgen")) {
                for (int n8 = 0; n8 < list3.size(); ++n8) {
                    int unsignedShort8 = nextChunk.readUnsignedShort();
                    int short2 = nextChunk.readShort();
                    SF2InstrumentRegion sf2InstrumentRegion = (SF2InstrumentRegion)list3.get(n8);
                    if (sf2InstrumentRegion != null) {
                        sf2InstrumentRegion.generators.put(Integer.valueOf(unsignedShort8), Short.valueOf((short)short2));
                    }
                }
            }
            else if (format.equals("inst")) {
                if (nextChunk.available() % 22 != 0) {
                    throw new RIFFInvalidDataException();
                }
                for (int n6 = nextChunk.available() / 22, n7 = 0; n7 < n6; ++n7) {
                    SF2Layer o4 = new SF2Layer(this);
                    o4.name = nextChunk.readString(20);
                    list6.add(Integer.valueOf(nextChunk.readUnsignedShort()));
                    list5.add(o4);
                    if (n7 != n6 - 1) {
                        this.layers.add(o4);
                    }
                }
            }
            else if (format.equals("ibag")) {
                if (nextChunk.available() % 4 != 0) {
                    throw new RIFFInvalidDataException();
                }
                int n2 = nextChunk.available() / 4;
                int unsignedShort2 = nextChunk.readUnsignedShort();
                int unsignedShort3 = nextChunk.readUnsignedShort();
                while (list7.size() < unsignedShort2) {
                    list7.add(null);
                }
                while (list8.size() < unsignedShort3) {
                    list8.add(null);
                }
                --n2;
                if (list6.isEmpty()) {
                    throw new RIFFInvalidDataException();
                }
                int intValue = (int)list6.get(0);
                int l = 0;
                while (l < intValue) {
                    if (n2 == 0) {
                        throw new RIFFInvalidDataException();
                    }
                    int unsignedShort4 = nextChunk.readUnsignedShort();
                    int unsignedShort5 = nextChunk.readUnsignedShort();
                    while (list7.size() < unsignedShort4) {
                        list7.add(null);
                    }
                    while (list8.size() < unsignedShort5) {
                        list8.add(null);
                    }
                    --n2;
                    ++l;
                }
                for (int n3 = 0; n3 < list6.size() - 1; ++n3) {
                    int n4 = (int)list6.get(n3 + 1) - (int)list6.get(n3);
                    SF2Layer sf2Layer = (SF2Layer)this.layers.get(n3);
                    int n5 = 0;
                    while (n5 < n4) {
                        if (n2 == 0) {
                            throw new RIFFInvalidDataException();
                        }
                        int unsignedShort6 = nextChunk.readUnsignedShort();
                        int unsignedShort7 = nextChunk.readUnsignedShort();
                        SF2LayerRegion o3 = new SF2LayerRegion();
                        sf2Layer.regions.add(o3);
                        while (list7.size() < unsignedShort6) {
                            list7.add(o3);
                        }
                        while (list8.size() < unsignedShort7) {
                            list8.add(o3);
                        }
                        --n2;
                        ++n5;
                    }
                }
            }
            else if (format.equals("imod")) {
                int k = 0;
                while (k < list8.size()) {
                    SF2Modulator o2 = new SF2Modulator();
                    o2.sourceOperator = nextChunk.readUnsignedShort();
                    o2.destinationOperator = nextChunk.readUnsignedShort();
                    o2.amount = nextChunk.readShort();
                    o2.amountSourceOperator = nextChunk.readUnsignedShort();
                    o2.transportOperator = nextChunk.readUnsignedShort();
                    if (k < 0 || k >= list7.size()) {
                        throw new RIFFInvalidDataException();
                    }
                    SF2LayerRegion sf2LayerRegion2 = (SF2LayerRegion)list7.get(k);
                    if (sf2LayerRegion2 != null) {
                        sf2LayerRegion2.modulators.add(o2);
                    }
                    ++k;
                }
            }
            else if (format.equals("igen")) {
                for (int j = 0; j < list7.size(); ++j) {
                    int unsignedShort = nextChunk.readUnsignedShort();
                    int short1 = nextChunk.readShort();
                    SF2LayerRegion sf2LayerRegion = (SF2LayerRegion)list7.get(j);
                    if (sf2LayerRegion != null) {
                        sf2LayerRegion.generators.put(Integer.valueOf(unsignedShort), Short.valueOf((short)short1));
                    }
                }
            }
            else if (format.equals("shdr")) {
                if (nextChunk.available() % 46 != 0) {
                    throw new RIFFInvalidDataException();
                }
                for (int n = nextChunk.available() / 46, i = 0; i < n; ++i) {
                    SF2Sample o = new SF2Sample(this);
                    o.name = nextChunk.readString(20);
                    long unsignedInt = nextChunk.readUnsignedInt();
                    long unsignedInt2 = nextChunk.readUnsignedInt();
                    if (this.sampleData != null) {
                        o.data = this.sampleData.subbuffer(unsignedInt * 2L, unsignedInt2 * 2L, true);
                    }
                    if (this.sampleData24 != null) {
                        o.data24 = this.sampleData24.subbuffer(unsignedInt, unsignedInt2, true);
                    }
                    o.startLoop = nextChunk.readUnsignedInt() - unsignedInt;
                    o.endLoop = nextChunk.readUnsignedInt() - unsignedInt;
                    if (o.startLoop < 0L) {
                        o.startLoop = -1L;
                    }
                    if (o.endLoop < 0L) {
                        o.endLoop = -1L;
                    }
                    o.sampleRate = nextChunk.readUnsignedInt();
                    o.originalPitch = nextChunk.readUnsignedByte();
                    o.pitchCorrection = nextChunk.readByte();
                    o.sampleLink = nextChunk.readUnsignedShort();
                    o.sampleType = nextChunk.readUnsignedShort();
                    if (i != n - 1) {
                        this.samples.add(o);
                    }
                }
            }
            else {
                continue;
            }
        }
        for (SF2Layer sf2Layer2 : this.layers) {
            Iterator<SF2LayerRegion> iterator2 = sf2Layer2.regions.iterator();
            SF2LayerRegion sf2LayerRegion3 = null;
            while (iterator2.hasNext()) {
                SF2LayerRegion sf2LayerRegion4 = (SF2LayerRegion)iterator2.next();
                if (sf2LayerRegion4.generators.get(Integer.valueOf(53)) != null) {
                    int shortValue = (short)sf2LayerRegion4.generators.get(Integer.valueOf(53));
                    sf2LayerRegion4.generators.remove(Integer.valueOf(53));
                    if (shortValue < 0 || shortValue >= this.samples.size()) {
                        throw new RIFFInvalidDataException();
                    }
                    sf2LayerRegion4.sample = (SF2Sample)this.samples.get(shortValue);
                }
                else {
                    sf2LayerRegion3 = sf2LayerRegion4;
                }
            }
            if (sf2LayerRegion3 != null) {
                sf2Layer2.getRegions().remove(sf2LayerRegion3);
                SF2GlobalRegion globalZone = new SF2GlobalRegion();
                globalZone.generators = sf2LayerRegion3.generators;
                globalZone.modulators = sf2LayerRegion3.modulators;
                sf2Layer2.setGlobalZone(globalZone);
            }
            else {
                continue;
            }
        }
        for (SF2Instrument sf2Instrument2 : this.instruments) {
            Iterator<SF2InstrumentRegion> iterator4 = sf2Instrument2.regions.iterator();
            SF2InstrumentRegion sf2InstrumentRegion3 = null;
            while (iterator4.hasNext()) {
                SF2InstrumentRegion sf2InstrumentRegion4 = (SF2InstrumentRegion)iterator4.next();
                if (sf2InstrumentRegion4.generators.get(Integer.valueOf(41)) != null) {
                    int shortValue2 = (short)sf2InstrumentRegion4.generators.get(Integer.valueOf(41));
                    sf2InstrumentRegion4.generators.remove(Integer.valueOf(41));
                    if (shortValue2 < 0 || shortValue2 >= this.layers.size()) {
                        throw new RIFFInvalidDataException();
                    }
                    sf2InstrumentRegion4.layer = (SF2Layer)this.layers.get(shortValue2);
                }
                else {
                    sf2InstrumentRegion3 = sf2InstrumentRegion4;
                }
            }
            if (sf2InstrumentRegion3 != null) {
                sf2Instrument2.getRegions().remove(sf2InstrumentRegion3);
                SF2GlobalRegion globalZone2 = new SF2GlobalRegion();
                globalZone2.generators = sf2InstrumentRegion3.generators;
                globalZone2.modulators = sf2InstrumentRegion3.modulators;
                sf2Instrument2.setGlobalZone(globalZone2);
            }
            else {
                continue;
            }
        }
    }
    
    public void save(String s) throws IOException {
        this.writeSoundbank(new RIFFWriter(s, "sfbk"));
    }
    
    public void save(File file) throws IOException {
        this.writeSoundbank(new RIFFWriter(file, "sfbk"));
    }
    
    public void save(OutputStream outputStream) throws IOException {
        this.writeSoundbank(new RIFFWriter(outputStream, "sfbk"));
    }
    
    private void writeSoundbank(RIFFWriter riffWriter) throws IOException {
        this.writeInfo(riffWriter.writeList("INFO"));
        this.writeSdtaChunk(riffWriter.writeList("sdta"));
        this.writePdtaChunk(riffWriter.writeList("pdta"));
        riffWriter.close();
    }
    
    private void writeInfoStringChunk(RIFFWriter riffWriter, String s, String s2) throws IOException {
        if (s2 == null) {
            return;
        }
        RIFFWriter writeChunk = riffWriter.writeChunk(s);
        writeChunk.writeString(s2);
        int length = s2.getBytes("ascii").length;
        writeChunk.write(0);
        if (++length % 2 != 0) {
            writeChunk.write(0);
        }
        return;
    }
    
    private void writeInfo(RIFFWriter riffWriter) throws IOException {
        if (this.targetEngine == null) {
            this.targetEngine = "EMU8000";
        }
        if (this.name == null) {
            this.name = "";
        }
        RIFFWriter writeChunk = riffWriter.writeChunk("ifil");
        writeChunk.writeUnsignedShort(this.major);
        writeChunk.writeUnsignedShort(this.minor);
        this.writeInfoStringChunk(riffWriter, "isng", this.targetEngine);
        this.writeInfoStringChunk(riffWriter, "INAM", this.name);
        this.writeInfoStringChunk(riffWriter, "irom", this.romName);
        if (this.romVersionMajor != -1) {
            RIFFWriter writeChunk2 = riffWriter.writeChunk("iver");
            writeChunk2.writeUnsignedShort(this.romVersionMajor);
            writeChunk2.writeUnsignedShort(this.romVersionMinor);
        }
        this.writeInfoStringChunk(riffWriter, "ICRD", this.creationDate);
        this.writeInfoStringChunk(riffWriter, "IENG", this.engineers);
        this.writeInfoStringChunk(riffWriter, "IPRD", this.product);
        this.writeInfoStringChunk(riffWriter, "ICOP", this.copyright);
        this.writeInfoStringChunk(riffWriter, "ICMT", this.comments);
        this.writeInfoStringChunk(riffWriter, "ISFT", this.tools);
        riffWriter.close();
    }
    
    private void writeSdtaChunk(RIFFWriter riffWriter) throws IOException {
        byte[] b = new byte[32];
        RIFFWriter writeChunk = riffWriter.writeChunk("smpl");
        Iterator<SF2Sample> iterator = this.samples.iterator();
        while (iterator.hasNext()) {
            ((SF2Sample)iterator.next()).getDataBuffer().writeTo(writeChunk);
            writeChunk.write(b);
            writeChunk.write(b);
        }
        if (this.major < 2) {
            return;
        }
        if (this.major == 2 && this.minor < 4) {
            return;
        }
        Iterator<SF2Sample> iterator2 = this.samples.iterator();
        while (iterator2.hasNext()) {
            if (((SF2Sample)iterator2.next()).getData24Buffer() == null) {
                return;
            }
            continue;
        }
        RIFFWriter writeChunk2 = riffWriter.writeChunk("sm24");
        Iterator<SF2Sample> iterator3 = this.samples.iterator();
        while (iterator3.hasNext()) {
            ((SF2Sample)iterator3.next()).getData24Buffer().writeTo(writeChunk2);
            writeChunk.write(b);
        }
        return;
    }
    
    private void writeModulators(RIFFWriter riffWriter, List<SF2Modulator> list) throws IOException {
        for (SF2Modulator sf2Modulator : list) {
            riffWriter.writeUnsignedShort(sf2Modulator.sourceOperator);
            riffWriter.writeUnsignedShort(sf2Modulator.destinationOperator);
            riffWriter.writeShort(sf2Modulator.amount);
            riffWriter.writeUnsignedShort(sf2Modulator.amountSourceOperator);
            riffWriter.writeUnsignedShort(sf2Modulator.transportOperator);
        }
    }
    
    private void writeGenerators(RIFFWriter riffWriter, Map<Integer, Short> map) throws IOException {
        Short n = (Short)map.get(Integer.valueOf(43));
        Short n2 = (Short)map.get(Integer.valueOf(44));
        if (n != null) {
            riffWriter.writeUnsignedShort(43);
            riffWriter.writeShort((short)n);
        }
        if (n2 != null) {
            riffWriter.writeUnsignedShort(44);
            riffWriter.writeShort((short)n2);
        }
        for (Map.Entry entry : map.entrySet()) {
            if ((int)entry.getKey() == 43) {
                continue;
            }
            if ((int)entry.getKey() == 44) {
                continue;
            }
            riffWriter.writeUnsignedShort((int)entry.getKey());
            riffWriter.writeShort((short)entry.getValue());
        }
    }
    
    private void writePdtaChunk(RIFFWriter riffWriter) throws IOException {
        RIFFWriter writeChunk = riffWriter.writeChunk("phdr");
        int n = 0;
        for (SF2Instrument sf2Instrument : this.instruments) {
            writeChunk.writeString(sf2Instrument.name, 20);
            writeChunk.writeUnsignedShort(sf2Instrument.preset);
            writeChunk.writeUnsignedShort(sf2Instrument.bank);
            writeChunk.writeUnsignedShort(n);
            if (sf2Instrument.getGlobalRegion() != null) {
                ++n;
            }
            n += sf2Instrument.getRegions().size();
            writeChunk.writeUnsignedInt(sf2Instrument.library);
            writeChunk.writeUnsignedInt(sf2Instrument.genre);
            writeChunk.writeUnsignedInt(sf2Instrument.morphology);
        }
        writeChunk.writeString("EOP", 20);
        writeChunk.writeUnsignedShort(0);
        writeChunk.writeUnsignedShort(0);
        writeChunk.writeUnsignedShort(n);
        writeChunk.writeUnsignedInt(0L);
        writeChunk.writeUnsignedInt(0L);
        writeChunk.writeUnsignedInt(0L);
        RIFFWriter writeChunk2 = riffWriter.writeChunk("pbag");
        int n2 = 0;
        int n3 = 0;
        for (SF2Instrument sf2Instrument2 : this.instruments) {
            if (sf2Instrument2.getGlobalRegion() != null) {
                writeChunk2.writeUnsignedShort(n2);
                writeChunk2.writeUnsignedShort(n3);
                n2 += sf2Instrument2.getGlobalRegion().getGenerators().size();
                n3 += sf2Instrument2.getGlobalRegion().getModulators().size();
            }
            for (SF2InstrumentRegion sf2InstrumentRegion : sf2Instrument2.getRegions()) {
                writeChunk2.writeUnsignedShort(n2);
                writeChunk2.writeUnsignedShort(n3);
                if (this.layers.indexOf(sf2InstrumentRegion.layer) != -1) {
                    ++n2;
                }
                n2 += sf2InstrumentRegion.getGenerators().size();
                n3 += sf2InstrumentRegion.getModulators().size();
            }
        }
        writeChunk2.writeUnsignedShort(n2);
        writeChunk2.writeUnsignedShort(n3);
        RIFFWriter writeChunk3 = riffWriter.writeChunk("pmod");
        for (SF2Instrument sf2Instrument3 : this.instruments) {
            if (sf2Instrument3.getGlobalRegion() != null) {
                this.writeModulators(writeChunk3, sf2Instrument3.getGlobalRegion().getModulators());
            }
            Iterator<SF2InstrumentRegion> iterator5 = sf2Instrument3.getRegions().iterator();
            while (iterator5.hasNext()) {
                this.writeModulators(writeChunk3, ((SF2InstrumentRegion)iterator5.next()).getModulators());
            }
        }
        writeChunk3.write(new byte[10]);
        RIFFWriter writeChunk4 = riffWriter.writeChunk("pgen");
        for (SF2Instrument sf2Instrument4 : this.instruments) {
            if (sf2Instrument4.getGlobalRegion() != null) {
                this.writeGenerators(writeChunk4, sf2Instrument4.getGlobalRegion().getGenerators());
            }
            for (SF2InstrumentRegion sf2InstrumentRegion2 : sf2Instrument4.getRegions()) {
                this.writeGenerators(writeChunk4, sf2InstrumentRegion2.getGenerators());
                int index = this.layers.indexOf(sf2InstrumentRegion2.layer);
                if (index != -1) {
                    writeChunk4.writeUnsignedShort(41);
                    writeChunk4.writeShort((short)index);
                }
                else {
                    continue;
                }
            }
        }
        writeChunk4.write(new byte[4]);
        RIFFWriter writeChunk5 = riffWriter.writeChunk("inst");
        int n4 = 0;
        for (SF2Layer sf2Layer : this.layers) {
            writeChunk5.writeString(sf2Layer.name, 20);
            writeChunk5.writeUnsignedShort(n4);
            if (sf2Layer.getGlobalRegion() != null) {
                ++n4;
            }
            n4 += sf2Layer.getRegions().size();
        }
        writeChunk5.writeString("EOI", 20);
        writeChunk5.writeUnsignedShort(n4);
        RIFFWriter writeChunk6 = riffWriter.writeChunk("ibag");
        int n5 = 0;
        int n6 = 0;
        for (SF2Layer sf2Layer2 : this.layers) {
            if (sf2Layer2.getGlobalRegion() != null) {
                writeChunk6.writeUnsignedShort(n5);
                writeChunk6.writeUnsignedShort(n6);
                n5 += sf2Layer2.getGlobalRegion().getGenerators().size();
                n6 += sf2Layer2.getGlobalRegion().getModulators().size();
            }
            for (SF2LayerRegion sf2LayerRegion : sf2Layer2.getRegions()) {
                writeChunk6.writeUnsignedShort(n5);
                writeChunk6.writeUnsignedShort(n6);
                if (this.samples.indexOf(sf2LayerRegion.sample) != -1) {
                    ++n5;
                }
                n5 += sf2LayerRegion.getGenerators().size();
                n6 += sf2LayerRegion.getModulators().size();
            }
        }
        writeChunk6.writeUnsignedShort(n5);
        writeChunk6.writeUnsignedShort(n6);
        RIFFWriter writeChunk7 = riffWriter.writeChunk("imod");
        for (SF2Layer sf2Layer3 : this.layers) {
            if (sf2Layer3.getGlobalRegion() != null) {
                this.writeModulators(writeChunk7, sf2Layer3.getGlobalRegion().getModulators());
            }
            Iterator<SF2LayerRegion> iterator12 = sf2Layer3.getRegions().iterator();
            while (iterator12.hasNext()) {
                this.writeModulators(writeChunk7, ((SF2LayerRegion)iterator12.next()).getModulators());
            }
        }
        writeChunk7.write(new byte[10]);
        RIFFWriter writeChunk8 = riffWriter.writeChunk("igen");
        for (SF2Layer sf2Layer4 : this.layers) {
            if (sf2Layer4.getGlobalRegion() != null) {
                this.writeGenerators(writeChunk8, sf2Layer4.getGlobalRegion().getGenerators());
            }
            for (SF2LayerRegion sf2LayerRegion2 : sf2Layer4.getRegions()) {
                this.writeGenerators(writeChunk8, sf2LayerRegion2.getGenerators());
                int index2 = this.samples.indexOf(sf2LayerRegion2.sample);
                if (index2 != -1) {
                    writeChunk8.writeUnsignedShort(53);
                    writeChunk8.writeShort((short)index2);
                }
                else {
                    continue;
                }
            }
        }
        writeChunk8.write(new byte[4]);
        RIFFWriter writeChunk9 = riffWriter.writeChunk("shdr");
        long n7 = 0L;
        for (SF2Sample sf2Sample : this.samples) {
            writeChunk9.writeString(sf2Sample.name, 20);
            long n8 = n7 + sf2Sample.data.capacity() / 2L;
            long n9 = sf2Sample.startLoop + n7;
            long n10 = sf2Sample.endLoop + n7;
            if (n9 < n7) {
                n9 = n7;
            }
            if (n10 > n8) {
                n10 = n8;
            }
            writeChunk9.writeUnsignedInt(n7);
            writeChunk9.writeUnsignedInt(n8);
            writeChunk9.writeUnsignedInt(n9);
            writeChunk9.writeUnsignedInt(n10);
            writeChunk9.writeUnsignedInt(sf2Sample.sampleRate);
            writeChunk9.writeUnsignedByte(sf2Sample.originalPitch);
            writeChunk9.writeByte((int)sf2Sample.pitchCorrection);
            writeChunk9.writeUnsignedShort(sf2Sample.sampleLink);
            writeChunk9.writeUnsignedShort(sf2Sample.sampleType);
            n7 = n8 + 32L;
        }
        writeChunk9.writeString("EOS", 20);
        writeChunk9.write(new byte[26]);
    }
    
    @Override
    public String getName() {
        return this.name;
    }
    
    @Override
    public String getVersion() {
        return this.major + "." + this.minor;
    }
    
    @Override
    public String getVendor() {
        return this.engineers;
    }
    
    @Override
    public String getDescription() {
        return this.comments;
    }
    
    public void setName(String name) {
        this.name = name;
    }
    
    public void setVendor(String engineers) {
        this.engineers = engineers;
    }
    
    public void setDescription(String comments) {
        this.comments = comments;
    }
    
    @Override
    public SoundbankResource[] getResources() {
        SoundbankResource[] array = new SoundbankResource[this.layers.size() + this.samples.size()];
        int n = 0;
        for (int i = 0; i < this.layers.size(); ++i) {
            array[n++] = (SoundbankResource)this.layers.get(i);
        }
        for (int j = 0; j < this.samples.size(); ++j) {
            array[n++] = (SoundbankResource)this.samples.get(j);
        }
        return array;
    }
    
    @Override
    public SF2Instrument[] getInstruments() {
        SF2Instrument[] a = (SF2Instrument[])this.instruments.toArray(new SF2Instrument[this.instruments.size()]);
        Arrays.sort(a, new ModelInstrumentComparator());
        return a;
    }
    
    public SF2Layer[] getLayers() {
        return (SF2Layer[])this.layers.toArray(new SF2Layer[this.layers.size()]);
    }
    
    public SF2Sample[] getSamples() {
        return (SF2Sample[])this.samples.toArray(new SF2Sample[this.samples.size()]);
    }
    
    @Override
    public Instrument getInstrument(Patch patch) {
        int program = patch.getProgram();
        int bank = patch.getBank();
        int percussion = 0;
        if ((patch instanceof ModelPatch)) {
            percussion = ((ModelPatch)patch).isPercussion();
        }
        for (Instrument instrument : this.instruments) {
            Patch patch2 = instrument.getPatch();
            int program2 = patch2.getProgram();
            int bank2 = patch2.getBank();
            if (program == program2 && bank == bank2) {
                int percussion2 = 0;
                if ((patch2 instanceof ModelPatch)) {
                    percussion2 = ((ModelPatch)patch2).isPercussion();
                }
                if (percussion == percussion2) {
                    return instrument;
                }
                continue;
            }
            else {
                continue;
            }
        }
        return null;
    }
    
    public String getCreationDate() {
        return this.creationDate;
    }
    
    public void setCreationDate(String creationDate) {
        this.creationDate = creationDate;
    }
    
    public String getProduct() {
        return this.product;
    }
    
    public void setProduct(String product) {
        this.product = product;
    }
    
    public String getRomName() {
        return this.romName;
    }
    
    public void setRomName(String romName) {
        this.romName = romName;
    }
    
    public int getRomVersionMajor() {
        return this.romVersionMajor;
    }
    
    public void setRomVersionMajor(int romVersionMajor) {
        this.romVersionMajor = romVersionMajor;
    }
    
    public int getRomVersionMinor() {
        return this.romVersionMinor;
    }
    
    public void setRomVersionMinor(int romVersionMinor) {
        this.romVersionMinor = romVersionMinor;
    }
    
    public String getTargetEngine() {
        return this.targetEngine;
    }
    
    public void setTargetEngine(String targetEngine) {
        this.targetEngine = targetEngine;
    }
    
    public String getTools() {
        return this.tools;
    }
    
    public void setTools(String tools) {
        this.tools = tools;
    }
    
    public void addResource(SoundbankResource soundbankResource) {
        if ((soundbankResource instanceof SF2Instrument)) {
            this.instruments.add((SF2Instrument)soundbankResource);
        }
        if ((soundbankResource instanceof SF2Layer)) {
            this.layers.add((SF2Layer)soundbankResource);
        }
        if ((soundbankResource instanceof SF2Sample)) {
            this.samples.add((SF2Sample)soundbankResource);
        }
    }
    
    public void removeResource(SoundbankResource soundbankResource) {
        if ((soundbankResource instanceof SF2Instrument)) {
            this.instruments.remove((SoundbankResource)soundbankResource);
        }
        if ((soundbankResource instanceof SF2Layer)) {
            this.layers.remove((SoundbankResource)soundbankResource);
        }
        if ((soundbankResource instanceof SF2Sample)) {
            this.samples.remove((SoundbankResource)soundbankResource);
        }
    }
    
    public void addInstrument(SF2Instrument sf2Instrument) {
        this.instruments.add(sf2Instrument);
    }
    
    public void removeInstrument(SF2Instrument sf2Instrument) {
        this.instruments.remove(sf2Instrument);
    }
}
